country_acronyms = [
    "USA",  # United States of America
    "CHN",  # China
    "HKG",  # Hong Kong
    "IND",  # India
    "JPN",  # Japan
    "RUS",  # Russia
    "SAU",  # Saudi Arabia
    "SIN",  # Singapore
    "KOR",  # South Korea
    "SWE",  # Sweden
    "CHE",  # Switzerland
    "ROC",  # Taiwan
    "UKR",  # Ukraine
    "DEU",  #
    "FRA",  # France
    "ARG",  # Argentina  
    "ITA",  # Italy
    "NZL",  # New Zealand
    "AUS",  # Australia
    "CAN",  # Canada
    "EUR",  # Europe

]

repo_codes = [
    "RRPTSYD",  # Treasury Securities Sold by the Federal Reserve
    "RPTSYD",  # Treasury Securities Purchased by the Federal Reserve
    "RPTTLD",  # Total Securities Purchased by the Federal Reserve
    "RPTTLD",  # Overnight Repurchase Agreements: Mortgage Backed Securities Sold
    "RPONMBSD",  # Overnight Repurchase Agreements: Mortgage-Backed Securities Purchased
    "RPONTSYD",  # Overnight Repurchase Agreements: Treasury Securities Purchased
    "RPONAGYD",  # Overnight Repurchase Agreements: Federal Agency Securities Purchased
    "RPMBSD",  # Repurchase Agreements: Mortgage-Backed Securities Purchased
    "RPAGYD",  # Repurchase Agreements: Federal Agency Securities Sold
    "RRPTTLD",  # Repurchase Agreements: Federal Agency Securities Purchased
    "RRPONTTLD",  # Reverse Repurchase Agreements: Total Securities Sold
]

sp500_codes = [
    "SP500_DIV_YIELD_MONTH",  # S&P 500 Dividend Yield by Month
    "SP500_PE_RATIO_MONTH",  # Monthly P/E Ratio
    "SHILLER_PE_RATIO_YEAR",  # Shiller PE Ratio by Year
    "SP500_PBV_RATIO_QUARTER",  # S&P 500 Price to Book Value by Quarter
    "SP500_REAL_PRICE_MONTH",  # S&P 500 Real Price by Month
    "SP500_INFLADJ_MONTH",  # S&P 500 Inflation Adjusted by Month
    "SP500_INFLADJ_YEAR",  # S&P 500 Inflation Adjusted by Year
    "SP500_BVPS_QUARTER",  # S&P 500 Book Value Per Share by Quarter
    "SP500_EARNINGS_GROWTH_QUARTER",  # S&P 500 Earnings Growth Rate by Quarter
    "SP500_SALES_QUARTER"  # S&P 500 Sales by Quarter
]


income_expenditure_codes = [
    "MEHOINUSA672N",  # Real Median Household Income in the United States
    "DSPIC96",  # Real Disposable Personal Income
    "PCE",  # Personal Consumption Expenditures
    "PCEDG",  # Personal Consumption Expenditures: Durable Goods
    "PSAVERT",  # Personal Saving Rate
    "RRSFS",  # Real Retail and Food Services Sales
    "DSPI"  # Disposable Personal Income
]


economic_indicator_codes = [ 

 
    "INDPRO",  # Industrial Production Index
    "TCU",  # Capacity Utilization: Total Industry
    "HOUST",  # Housing Starts: Total: New Privately Owned Housing Units Started
    "GPDI",  # Gross Private Domestic Investment
    "cp",  # Corporate Profits After Tax (without IVA and CCAdj)
    "STLFSI",  # St. Louis Fed Financial Stress Index
    "DCOILWTICO",  # Crude Oil Prices: West Texas Intermediate (WTI)
    "USSLIND",  # Leading Index for the United States
    "DTWEXM",  # Trade Weighted U.S. Dollar Index: Major Currencies
    "DTWEXB"  # Trade Weighted U.S. Dollar Index: Broad

]

unemployment_codes = [
    "UNRATE",  # Civilian Unemployment Rate
    "NROU",  # Natural Rate of Unemployment (Long-Term)
    "NROUST",  # Natural Rate of Unemployment (Short-Term)
    "CIVPART",  # Civilian Labor Force Participation Rate
    "EMRATIO",  # Civilian Employment-Population Ratio
    "UNEMPLOY",  # Unemployed
    "PAYEMS",  # All Employees: Total nonfarm
    "MANEMP",  # All Employees: Manufacturing
    "ICSA",  # Initial Claims
    "IC4WSA"  # 4-Week Moving Average of Initial Claims
]


interest_rate_codes = [
    "DFF",  # Effective Federal Funds Rate
    "DTB3",  # 3-Month Treasury Bill: Secondary Market Rate
    "DGS5",  # 5-Year Treasury Constant Maturity Rate
    "DGS10",  # 10-Year Treasury Constant Maturity Rate
    "DGS30",  # 30-Year Treasury Constant Maturity Rate
    "T5YIE",  # 5-year Breakeven Inflation Rate
    "T10YIE",  # 10-year Breakeven Inflation Rate
    "T5YIFR",  # 5-Year, 5-Year Forward Inflation Expectation Rate
    "TEDRATE",  # TED Spread
    "DPRIME"  # Bank Prime Loan Rate
]

gdp_codes = [
    "GDP",  # Gross Domestic Product
    "GDPC1", # Real Gross Domestic Product
    "GDPPOT" 
]